#!/bin/csh -f

#
# make a shifted pair of images
#
set path = (/Volumes/CARVE/tools-src $path)
if ($#argv == 0) then
	echo "Usage: prefix data-set-path"
	exit 0
endif
#
# setup file names
#
set prefix = $1
set dir = $2
set suffix = `date +_%y%m%d%H%M`
set img = /tmp/$prefix-shifted$suffix.dd
set alt = /tmp/$prefix-notshifted$suffix.dd
set log = /tmp/$prefix-shifted$suffix-log.txt
set dlist = (`ls /tmp | grep $prefix | grep shifted`)

#
# delete old images
#
if ($#dlist > 0) then
	echo "Delete $dlist"
	foreach f ($dlist)
		/bin/rm /tmp/$f
	end
endif
touch $log

# set fillers
set fill_dir = "/Volumes/CARVE/filler/"
set fill_files = (`ls /Volumes/CARVE/filler | grep -v profile`)
#
foreach file ($fill_files)
	if (-e /tmp/$file) /bin/rm /tmp/$file
	cp ${fill_dir}$file /tmp/$file
	cat ${fill_dir}$file ${fill_dir}$file ${fill_dir}$file >> /tmp/$file
	cat ${fill_dir}$file ${fill_dir}$file ${fill_dir}$file >> /tmp/$file
	cat ${fill_dir}$file >> /tmp/$file
end


dd bs=512 count=1 if=/dev/zero of=$img >& /dev/null
dd bs=512 count=1 if=/dev/zero of=$alt >& /dev/null


set tab = /tmp/$prefix-shifted$suffix-offsets.txt
echo "Shift log for $img and $alt" > $tab
echo "<start-table>" >> $tab
echo "Embedded File Offsets;" >> $tab
echo "File;Offset in Bytes" >> $tab
set size_list = (1 2 4 8 16 32 64 128)
set off_list = (1 511 4 510 16 496 64 128)
set fill_list = ($fill_files)
set exts_avail = (`/bin/ls $dir | sed -e 's/.*\.//' | tr '[A-Z]' '[a-z]' | sort -u | egrep -v '(txt)|(csh)|(dd)'`)
set f_list = (`ls $dir | egrep -v '(\.txt)|(\.csh)|(\.dd)'`)
echo "Fill: " $fill_list >> $log
echo "=======================" >> $log
echo "Extensions: " $exts_avail >> $log
echo "=======================" >> $log
#echo "Files: " $f_list >> $log
foreach xxx ($exts_avail)
set f_list = (`ls $dir | grep -i "\.$xxx" | head -15`)
@ cnt = 0
echo "=======================" >> $log
echo "Files ($xxx): " $f_list >> $log
echo "=======================" >> $log
echo "Processing $xxx"
foreach fx ($f_list)

	set file = $dir/$fx
	@ ix = ($cnt % $#size_list) + 1
	@ jx = ($cnt % $#fill_list) + 1

	echo "$fx (byte offset $off_list[$ix]) followed by $size_list[$ix] sectors of $fill_list[$jx]" >> $log
	echo "$fx;$off_list[$ix]" >> $tab
	block-cat.py $file >> $alt
	block-cat.py $file > /tmp/not-shifted.dd
	(dd if=/tmp/$fill_list[$jx] bs=512 count=$size_list[$ix] >> $alt) >& /dev/null
	(dd if=/tmp/$fill_list[$jx] bs=1 count=$off_list[$ix] > /tmp/shifted.dd) >& /dev/null
	cat $file >> /tmp/shifted.dd
	block-cat.py /tmp/shifted.dd >> $img
	block-cat.py /tmp/shifted.dd > /tmp/shifted-blocked.dd
	(dd if=/tmp/$fill_list[$jx] bs=512 count=$size_list[$ix] >> $img) >& /dev/null
	set shifted_size = `ls -l /tmp/shifted-blocked.dd | awk '{print $5;}'`
	set not_shifted_size = `ls -l /tmp/not-shifted.dd | awk '{print $5;}'`
	if ($shifted_size == $not_shifted_size) then
		echo "add $fx ($not_shifted_size)" >> /tmp/shift-log.txt
	else
		@ sb = $shifted_size
		@ not_sb = $not_shifted_size
		@ offset = $sb - $not_sb
		echo "add $fx ($not_shifted_size) with shift ($shifted_size) $offset more bytes" >> /tmp/shift-log.txt
		dd bs=512 count=1 if=/dev/zero of=/tmp/one-zero.dd >& /dev/null
		cat /tmp/one-zero.dd >> $alt
	endif
	ls -l $file /tmp/not-shifted.dd /tmp/shifted.dd /tmp/shifted-blocked.dd >> /tmp/shift-log.txt

	@ cnt = $cnt + 1
end
end
echo "<end-table>" >> $tab
