#!/usr/bin/python
import hashlib
import sys

#	generate a set of block hashes as a profile of a set of files
#	output: 
#		unique hashes
#		hashes common across multiple files 
#		hashes for last sector filled out to a full sector
#		file summary
#

# sample output	

#	Summary bn.xxx 10 blocks 5120 bytes
#	Summary lav.xxx 10 blocks 5120 bytes
#	Summary rr.xxx 6 blocks 2925 bytes
#	Summary white.xxx 10 blocks 5120 bytes
#	Summary yellow.xxx 10 blocks 5120 bytes
#	Start File bn.xxx
#	H 0 28642ba995bfd672d3c0ff159dd7b26f
#	. . .
#	H 8 76b3c06f5be8b265b5da653de9be703d
#	H 9 6d59537db71f60e6493a38533f89b9ed
#	End File yellow.xxx
#	Common 1
#	CH d9e05b1d25f1e29e65de31001feba0be 0
#	CF 0 bn.xxx 5
#	CF 0 lav.xxx 5
#	CF 0 white.xxx 5
#	CF 0 yellow.xxx 5
#	tail
#	T 204afb3ac88334fb6a56d64b2ab7c22a rr.xxx 5
#	V @(#) profile-files.py Version 1.6 created 05/10/13 at 14:42:32



ver = "V @(#) profile-files.py Version 1.6 created 05/10/13 at 14:42:32"

bs = 512
unique_hashes = {}
common_hashes = {}
tail_hashes = {}
n_files = 0
n_common = 0
profile_list = []
t_blocks = 0

# examine each file
#
for file_name in sys.argv[1:]:
	n_dup = 0 # track number of duplicate blocks in a file
	n_files = n_files + 1
	f = open (file_name,"rb")
	block = f.read(bs)
	from_byte = 0
	n_bytes = 0
	blk_no = 0
	hash_list = {}
	fx = [] # list of block hashes in the file
	while True:
#		print "bn =",blk_no
		if len(block) == 0:
			n_blocks = blk_no
			break
		m = hashlib.md5()
		m.update(block)
		n_bytes = n_bytes + len(block)
		t_blocks = t_blocks + 1
		fx.append(m.hexdigest())
		if m.hexdigest() in hash_list: # check if this block appears elsewhere in this file
#			sys.stderr.write("Duplicate: Block %d of %s\n"%(blk_no,file_name.split('/')[-1]))
			n_dup += 1
			hash_list[m.hexdigest()].append(blk_no)
		else:
			hash_list[m.hexdigest()] = [blk_no]
#		print "len",len(block),blk_no
		if len(block) < bs:
			m.update((bs-len(block))*'\x00')
			if file_name.split('/')[-1] in tail_hashes:
#				print "Dup tail hash %s %s"%(file_name.split('/')[-1],m.hexdigest())
				n_dup += 1
#			tail_hashes[m.hexdigest()] = [file_name.split('/')[-1],blk_no]
			tail_hashes[file_name.split('/')[-1]] = [m.hexdigest(),blk_no]
#			print "Fix tail"
		block = f.read(bs)
		blk_no = blk_no + 1

#		***** End Loop to examine a file ****
# fx has a list of hashes for each block of the file
# hash_list has a list of unique hashes seen
# each hash in hash_list maps to a list of blocks with that hash
#



	profile_list.append([file_name.split('/')[-1],fx])
	nc = 0
#	print "\n\nProcess ",file_name.split('/')[-1]
	for h in hash_list:
		blk_list = hash_list[h]
#		if h in unique_hashes:
#			if h in common_hashes:
#				for blk_no in blk_list:
#					common_hashes[h].append((file_name.split('/')[-1],blk_no))
#			else:
#				common_hashes[h] = [n_common,unique_hashes[h]]
#				n_common = n_common + 1
#			if h in unique_hashes: del unique_hashes[h]
#		if h in common_hashes: print h," is common"
#		if h in unique_hashes: print h," is Unique"

		if (h in common_hashes) or (len(blk_list) > 1) or (h in unique_hashes):
			for blk_no in blk_list:
				if h in common_hashes:
#					print "H in Common, Before append: ",common_hashes[h]
#					print "Append: ",file_name.split('/')[-1],blk_no
					common_hashes[h].append((file_name.split('/')[-1],blk_no))
#					print "H in Common, After append: ",common_hashes[h]
				else:
#					print "Add",h,"To common",blk_no,file_name.split('/')[-1]
					common_hashes[h] = [n_common,(file_name.split('/')[-1],blk_no)]
#					print "After add: ",common_hashes[h]
					n_common = n_common + 1
				nc = nc + 1
			if h in unique_hashes:
#				common_hashes[h].append(unique_hashes[h])
#				print "Before append: ",common_hashes[h]
#				print "Unique: ",unique_hashes[h]
				common_hashes[h].append(unique_hashes[h])
#				common_hashes[h].append((file_name.split('/')[-1],blk_no))
#				print "Append to common:",unique_hashes[h],blk_no,file_name.split('/')[-1]
#				print "After append: ",common_hashes[h]
				del unique_hashes[h]
		else:
			unique_hashes[h] = (file_name.split('/')[-1],blk_list[0])
#			unique_hashes[h] = (file_name.split('/')[-1],blk_no)
#			print "Add",h,"To unique, block:",blk_no,blk_list
#
#
#
	print "Summary %s %d blocks %d bytes %d duplicate blocks"%(file_name.split('/')[-1],n_blocks,n_bytes,n_dup)
for file_profile in profile_list:
	at = 0
	print "Start File",file_profile[0]
	for bh in file_profile[1]:
		print "H",at,bh
		at = at + 1
	print "End File",file_profile[0]
print "Common",n_common
for x in common_hashes:
#	print "x = ",x,common_hashes[x]
	z = common_hashes[x]
	print "CH",x,z[0]
#	print "z",z
	for y in z[1:]:
		
		print "CF",z[0],y[0],
#		for yy in y[1]:
#			print yy,
		print y[1]
#		print "Trace ********: ",y

print "tail"
for t in tail_hashes:
#	print "T",t,tail_hashes[t][0],tail_hashes[t][1]
	print "T",tail_hashes[t][0],t,tail_hashes[t][1]
sys.stderr.write ("%d files, %d blocks hashed\n"%(n_files,t_blocks))
print ver
sys.stderr.write (ver+'\n')
